Texture2D inputTexture <string uiname="Texture";>;

uint activeViewport;
float3 safenormalize(float3 x)
{
	x=lerp(x,x+.00001,x==0);
	return normalize(x);
}

SamplerState g_samLinear <string uiname="Sampler State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Mirror;
    AddressV = Mirror;
};

cbuffer cbPerDraw : register( b0 )
{
	
	float4x4 tVP : VIEWPROJECTION;
	float4x4 tW : WORLD;
};

cbuffer cbPerObj : register( b1 )
{
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
};

struct VS_IN
{
	float4 PosO : POSITION;
	float4 TexCd : TEXCOORD0;
};

struct vs2ps
{
	uint viewport : SV_ViewportArrayIndex;
	float4 Pos: SV_POSITION;
	float4 Color : COLOR0;
	float4 TexCd: TEXCOORD0;
};

vs2ps VS(VS_IN input)
{
	vs2ps output;
	output.Pos = mul(input.PosO,mul(tW,tVP));
	output.TexCd = input.TexCd;
	output.Color = cAmb;
	return output;
}


[maxvertexcount(3)]
void GS(triangle vs2ps input[3], inout TriangleStream<vs2ps> gsout)
{
	vs2ps elem = (vs2ps)0;


    elem.viewport = activeViewport;
 
    for( int v = 0; v < 3; v++ )
    {
        elem.Pos = input[v].Pos;
        elem.Color = input[v].Color;
        gsout.Append( elem );
    }
	gsout.RestartStrip();
}

float4 PS(vs2ps In): SV_Target
{
    float4 col = inputTexture.Sample(g_samLinear,In.TexCd.xy) * cAmb;
    return col;
}


technique10 Visual
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader ( CompileShader( gs_5_0, GS() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
}




